#include <libxml/parser.h>
#include <libsoup/soup.h>
#include <glib/gstdio.h>
#include <glib/goption.h>
#include <gtk/gtk.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkwindow.h>

#define SERVER_URI "http://www.arkivarium.no:8092/noark5v5"

typedef struct {
    GtkWidget *widget;
    gint index;
    const gchar *title;
    GtkAssistantPageType type;
    gboolean complete;
} PageInfo;

// Widgets for entry fields
GtkWidget *role_entry;
GtkWidget *server_entry;
GtkWidget *login_entry;

GMainLoop *main_loop;

// Command-line options
static gint timeout = 3600;
static gint time_threshold = 0;

static GOptionEntry entries[] = {
    { "timeout", 't', 0, G_OPTION_ARG_INT, &timeout, "Exit after T seconds. Default: 3600", "T" },
    { "time-threshold", 'i', 0, G_OPTION_ARG_INT, &time_threshold, "Only report location update after T seconds. Default: 0", "T" },
    { NULL }
};

// Helper function to initialize Soup session and perform request
static void send_request(const gchar *uri) {
    SoupSession *session = soup_session_sync_new();
    SoupMessage *msg = soup_message_new("GET", uri);
    soup_message_set_request(msg, "application/xml", SOUP_MEMORY_COPY, g_strdup("arkivdel"), strlen("arkivdel"));
    soup_session_send_message(session, msg);
    g_object_unref(msg);
    g_object_unref(session);
}

// Assistant pages setup helper
static GtkWidget* create_assistant_page(GtkAssistant *assistant, PageInfo *page_info) {
    GtkWidget *page = gtk_box_new(GTK_ORIENTATION_VERTICAL, 10);
    gtk_assistant_append_page(assistant, page);
    gtk_assistant_set_page_title(assistant, page, page_info->title);
    gtk_assistant_set_page_type(assistant, page, page_info->type);
    gtk_assistant_set_page_complete(assistant, page, page_info->complete);
    return page;
}

// Function to handle assistant "apply" signal
static void on_assistant_apply(GtkAssistant *assistant, gpointer data) {
    send_request(SERVER_URI);
}

// Function to handle assistant "cancel" signal
static void on_assistant_cancel(GtkAssistant *assistant, gpointer data) {
    if (main_loop) {
        g_main_loop_quit(main_loop);
    }
    gtk_main_quit();
}

// Function to handle assistant "close" signal
static void on_assistant_close(GtkAssistant *assistant, gpointer data) {
    gtk_main_quit();
}

// Activate function to set up the GUI
static void activate(GtkApplication *app, gpointer user_data) {
    GtkWidget *assistant = gtk_assistant_new();
    gtk_window_set_application(GTK_WINDOW(assistant), app);
    gtk_window_set_title(GTK_WINDOW(assistant), "Arkivarium (nikita-gtk)");
    gtk_widget_set_size_request(assistant, 800, 600);

    // Page information
    PageInfo pages[] = {
        {NULL, -1, "Introduction", GTK_ASSISTANT_PAGE_INTRO, TRUE},
        {NULL, -1, "Role", GTK_ASSISTANT_PAGE_CONTENT, TRUE},
        {NULL, -1, "Server", GTK_ASSISTANT_PAGE_CONTENT, TRUE},
        {NULL, -1, "Login", GTK_ASSISTANT_PAGE_CONFIRM, TRUE}
    };

    // Introduction page
    pages[0].widget = gtk_label_new("Welcome to Arkivarium!\n\n");
    gtk_box_pack_start(GTK_BOX(create_assistant_page(GTK_ASSISTANT(assistant), &pages[0])), pages[0].widget, FALSE, FALSE, 0);

    // Role page
    GtkWidget *role_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
    GtkWidget *role_label = gtk_label_new("Role:");
    role_entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(role_box), role_label, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(role_box), role_entry, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(create_assistant_page(GTK_ASSISTANT(assistant), &pages[1])), role_box, FALSE, FALSE, 0);

    // Server page
    GtkWidget *server_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
    GtkWidget *server_label = gtk_label_new("Server:");
    server_entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(server_box), server_label, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(server_box), server_entry, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(create_assistant_page(GTK_ASSISTANT(assistant), &pages[2])), server_box, FALSE, FALSE, 0);

    // Login page
    GtkWidget *login_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
    GtkWidget *login_label = gtk_label_new("Login:");
    login_entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(login_box), login_label, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(login_box), login_entry, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(create_assistant_page(GTK_ASSISTANT(assistant), &pages[3])), login_box, FALSE, FALSE, 0);

    // Signal connections for assistant actions
    g_signal_connect(assistant, "apply", G_CALLBACK(on_assistant_apply), NULL);
    g_signal_connect(assistant, "cancel", G_CALLBACK(on_assistant_cancel), NULL);
    g_signal_connect(assistant, "close", G_CALLBACK(on_assistant_close), NULL);

    gtk_widget_show_all(assistant);
}

// Main function with command-line argument parsing
int main(int argc, char **argv) {
    GtkApplication *app;
    int status;
    GError *error = NULL;
    GOptionContext *context = g_option_context_new("- Arkivarium (nikita-gtk) 0.2.0");

    g_option_context_add_main_entries(context, entries, NULL);
    g_option_context_add_group(context, gtk_get_option_group(TRUE));

    if (!g_option_context_parse(context, &argc, &argv, &error)) {
        g_print("Option parsing failed: %s\n", error->message);
        g_error_free(error);
        return EXIT_FAILURE;
    }

    app = gtk_application_new("com.arkivarium.nikita_gtk", G_APPLICATION_FLAGS_NONE);
    g_signal_connect(app, "activate", G_CALLBACK(activate), NULL);

    status = g_application_run(G_APPLICATION(app), argc, argv);
    g_object_unref(app);
    return status;
}

